# Copyright (C) 2010 Andras Petrik <bikmak@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
from xl import settings
from xl.nls import gettext as _

class TrayButtons (object) :
    """
        Represents the plugin
    """
    
    
    def __init__(self, exaile):
        self.__exaile = exaile
        
        # Inicializing icons
        self.__tyNext = gtk.status_icon_new_from_stock("gtk-media-next")
        self.__tyNext.set_visible(False)
        self.__tyPause = gtk.status_icon_new_from_stock("gtk-media-pause")
        self.__tyPause.set_visible(False)
        self.__tyPrevious = gtk.status_icon_new_from_stock("gtk-media-previous")
        self.__tyPrevious.set_visible(False)
        self.__tyStop = gtk.status_icon_new_from_stock("gtk-media-stop")
        self.__tyStop.set_visible(False)
        
        self.__tyPrevious.connect("activate", self.on_previous, None)
        self.__tyPrevious.set_tooltip(_("Previous Track"))
        self.__tyPause.connect("activate", self.on_pause, None)
        self.__tyPause.set_tooltip(_("Play/Pause"))
        self.__tyNext.connect("activate", self.on_next, None)
        self.__tyNext.set_tooltip(_("Next Track"))
        self.__tyStop.connect("activate", self.on_stop, None)
        self.__tyStop.set_tooltip(_("Stop"))
        
        # Drawing icons in the correct order
        self.enable()
        
    def on_pause( self, par1, par2 ):
        """
            Called when toggle pause icon is clicked
        """
        if self.__exaile.player.is_paused() or self.__exaile.player.is_playing():
            self.__exaile.player.toggle_pause()
        else:
            self.__exaile.queue.play()
        self.set_pause_icon()

    def on_previous( self, par1, par2 ):
        """
            Called when previous icon is clicked
        """
        self.__exaile.queue.prev()
    
    def on_next( self, par1, par2 ):
        """
            Called when next icon is clicked
        """
        self.__exaile.queue.next()
        
    def on_stop( self, par1, par2 ):
        """
            Called when stop icon is clicked
        """
        self.__exaile.player.stop()
        
    def on_option_set(self, event, settings, option):
        """
            Called when change settings in the configuration panel
        """
        if option == 'plugin/traybuttons/show_track_information':
            self.set_next_tooltip()
            self.set_previous_tooltip()
        else:
            # needed for reordering icons
            self.redraw_icons()
        
    def disable(self):
        """
            Disable the plugin
        """
        self.__tyNext.set_visible(False)
        self.__tyPause.set_visible(False)
        self.__tyPrevious.set_visible(False)
        self.__tyStop.set_visible(False)
        
    def enable(self):
        """
            Enable the plugin
        """
        if self.icons_reversed:
            self.__tyNext.set_visible(self.next_visible)
            self.__tyPause.set_visible(self.pause_visible)
            self.__tyPrevious.set_visible(self.previous_visible)
            self.__tyStop.set_visible(self.stop_visible)
        else: 
            self.__tyStop.set_visible(self.stop_visible)
            self.__tyPrevious.set_visible(self.previous_visible)
            self.__tyPause.set_visible(self.pause_visible)
            self.__tyNext.set_visible(self.next_visible)
            
    def redraw_icons(self):
        """
            Redrawing icons
        """
        self.disable()
        self.enable()
        
    # ### Properties ### #
        
    @property    
    def pause_visible(self):
        return settings.get_option('plugin/traybuttons/pause_visible', True)
    
    @pause_visible.setter
    def pause_visible(self, value):
        settings.set_option('plugin/traybuttons/pause_visible', value)
        self.redraw_icons()
        
    @property    
    def previous_visible(self):
        return settings.get_option('plugin/traybuttons/previous_visible', True)
    
    @previous_visible.setter
    def previous_visible(self, value):
        settings.set_option('plugin/traybuttons/previous_visible', value)
        self.redraw_icons()
        
    @property    
    def next_visible(self):
        return settings.get_option('plugin/traybuttons/next_visible', True)
    
    @next_visible.setter
    def next_visible(self, value):
        settings.set_option('plugin/traybuttons/next_visible', value)
        self.redraw_icons()
        
    @property    
    def stop_visible(self):
        return settings.get_option('plugin/traybuttons/stop_visible', False)
    
    @stop_visible.setter
    def stop_visible(self, value):
        settings.set_option('plugin/traybuttons/stop_visible', value)
        self.redraw_icons()
        
    @property    
    def icons_reversed(self):
        return settings.get_option('plugin/traybuttons/icons_reversed', True)
    
    @icons_reversed.setter
    def icons_reversed(self, value):
        settings.set_option('plugin/traybuttons/icons_reversed', value)
        self.redraw_icons()
        
    @property    
    def show_track_information(self):
        return settings.get_option('plugin/traybuttons/show_track_information', False)
    
    @show_track_information.setter
    def show_track_information(self, value):
        settings.set_option('plugin/traybuttons/show_track_information', value)
        if ( value == False ) :
            self.__tyPrevious.set_tooltip(_("Previous Track"))
            self.__tyNext.set_tooltip(_("Next Track"))
    
    def set_previous_tooltip(self):
        """
            Set the tooltip of the previous icon
        """
        if self.show_track_information :
            tracks = self.__exaile.queue.current_playlist.get_tracks()
            current_pos = self.__exaile.queue.current_playlist.get_current_pos()
            if current_pos - 1  >= 0 :
                track = tracks[current_pos - 1]
                title = track.get_tag_display('title')
                artist = track.get_tag_display('artist', artist_compilations=False)
                self.__tyPrevious.set_tooltip(_("Previous track: %(title)s by %(artist)s") % {
                    'title' : title,
                    'artist' : artist
                })
            else :
                self.__tyPrevious.set_tooltip(_("Previous Track"))
        else :
            self.__tyPrevious.set_tooltip(_("Previous Track"))
    
    def set_next_tooltip(self):
        """
            Set the tooltip of the next icon
        """
        if self.show_track_information :
            tracks = self.__exaile.queue.current_playlist.get_tracks()
            current_pos = self.__exaile.queue.current_playlist.get_current_pos()
            if current_pos + 1  < len(tracks) :
                track = tracks[current_pos + 1]
                title = track.get_tag_display('title')
                artist = track.get_tag_display('artist', artist_compilations=False)
                self.__tyNext.set_tooltip(_("Next track: %(title)s by %(artist)s") % {
                    'title' : title,
                    'artist' : artist
                })
            else:
                self.__tyNext.set_tooltip(_("Next Track"))
        else:
            self.__tyNext.set_tooltip(_("Next Track"))
    
    def set_pause_icon(self, type) :
        """
            Set the play/pause icon
        """  
        if type == "playback_player_pause" or type == "playback_player_end" :
            self.__tyPause.set_from_stock( "gtk-media-play" )
            self.__tyPause.set_tooltip(_("Play"))
        else :
            self.__tyPause.set_from_stock( "gtk-media-pause" )
            self.__tyPause.set_tooltip(_("Pause"))
        
    def player_changed( self, eventType, player, data ):
        """
            Called on player events (start, stop, pause)
        """   
        self.set_pause_icon( eventType )
        
    def track_changed( self, eventType, player, track ):
        """
            Called when started to play a new track
        """
        self.__tyPause.set_from_stock( "gtk-media-pause" )
        self.__tyPause.set_tooltip(_("Pause"))
        if self.show_track_information :
            self.set_next_tooltip()
            self.set_previous_tooltip()
        else:
            self.__tyPrevious.set_tooltip(_("Previous Track"))
            self.__tyNext.set_tooltip(_("Next Track"))
        self.set_pause_icon(eventType)
        